#!/bin/bash

########################################
# Author: Bill Swords
# Script Name: autoupdate
  Version="0.0.9"
# Creation Date: 03/05/20 15:49:13
# Updated: 05/28/2023 12:47:06 PM     
# Description: Automatically installs system updates and removes unneeded files.
 changes="
 5-27-23 - v0.0.9 - Insert 24.04's code name and fix some bugs.
 3-30-23 - v0.0.8 - Added option to download current LTS version or Daily Build.
 3-30-23 - v0.0.8 - Added distribution upgrade check &amp; ability
 3-28-23 - v0.0.7 - Added new LTS OS availability check.
 3-21-23 - v0.0.6 - Added \"Reset Password\" and help options.
 3-16-23 - v0.0.5 - Added a work around for \"eggs\".
 Previous  v0.0.4 - Recontructed the counting mechanism."
########################################

txtrst=$(tput sgr0) # Text reset
txtred=$(tput setaf 1) # Red
txtgrn=$(tput setaf 2) # Green
txtylw=$(tput setaf 3) # Yellow
txtblu=$(tput setaf 4) # Blue
txtpur=$(tput setaf 5) # Purple
txtcyn=$(tput setaf 6) # Cyan
txtwht=$(tput setaf 7) # White

# ~~~~~~~~~~~~ START THE EDITS HERE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

usage() {
echo "   "
echo "   Bill's Auto Updater Version $Version"
echo "   usage: autoupdate.sh <-COMMAND> (-h for help)"
echo "   "
}

URL="www.lovinlinux.com"
# Let's figure out what version we have and what is available
# What is our surrent desktop?
myDesktop=$(echo $XDG_CURRENT_DESKTOP)
# What is the current stable build?
curr_year=$(date +%y)
curr_ISO=$(curl -L -s https://cdimage.ubuntu.com/ubuntu-mate/releases/ | grep $curr_year | tail -1 | cut -d "\"" -f 2 | cut -c 1-5)
# Gives us for example, 22.04
curr_ver=$(cat /etc/os-release | grep VERSION_ID | cut -d '"' -f 2)
# Take the year from the release
main_ver=$(echo $curr_ver | cut -d "." -f 1)
sub_ver=$(echo $curr_ver | cut -d "." -f 2)
# Now we add 2 to the year and see if it is divisible by 2. If so it's a LTS. Why this way? 
LTSyear=$(echo $(($main_ver + 2)))

#if [[ $(($main_ver % 2)) -eq 0 ]] && [[ "$sub_ver" -eq "04" ]];then
if [[ $(($main_ver % 2)) -eq 0 ]];then
    next_LTS=$(echo "scale=2; $curr_ver + 2" | bc )
fi
if [[ $(($curr_year % 2)) -ne 0 ]];then
    last_LTS=$(echo "scale=2; $curr_year - 1" | bc ).04
fi

# Set some vairables for the different versions.\
        if [ "$curr_ver" == "20.04" ];then
            codename="Focal Fossa"
            next_codename="Jammy Jellyfish"
        elif [ "$curr_ver" == "20.10" ];then
            codename="Groovy Gorilla"
            next_codename="Jammy Jellyfish"
        elif [ "$curr_ver" == "21.04" ];then
            codename="Hirsute Hippo"
            next_codename="Jammy Jellyfish"
        elif [ "$curr_ver" == "21.10" ];then
            codename="Impish Indri"
            next_codename="Jammy Jellyfish"
        elif [ "$curr_ver" == "22.04" ];then
            codename="Jammy Jellyfish"
            next_codename="Naughty Nightingale"
        elif [ "$curr_ver" == "22.10" ];then
            codename="Kinetic Kudu"
            next_codename="Naughty Nightingale"
        elif [ "$curr_ver" == "23.04" ];then
            codename="Lunar Lobster"
            next_codename="Naughty Nightingale"
        elif [ "$curr_ver" == "23.10" ];then
            codename="Mantic Minotaur"
            next_codename="Naughty Nightingale"
        elif [ "$curr_ver" == "24.04" ];then
            codename="Naughty Nightingale"
            next_codename="To Be Announced"
        fi

reset_pass() {
    if [ ! -f "$HOME/.config/.autoupdate" ]; then
        
        MyPass=$(zenity --password --title="Password Required")
            if [ -z "$MyPass" ];then
                zenity --error --title="Password Required" --text="Password seems to be empty. Exiting now. \nPlease rerun the program and enter a valid password." --ellipsize
                exit 0
            else
                touch $HOME/.config/.autoupdate
                echo $MyPass > $HOME/.config/.autoupdate
        zenity --info --timeout=2 --title="Password Remembered" --text="Ok, I got it. I'll remember that password for you." --ellipsize
            fi
            
    elif  [ -f "$HOME/.config/.autoupdate" ]; then
    
        MyPass=$(zenity --password --title="Password Required")
            if [ -z "$MyPass" ];then
                zenity --error --title="Password Required" --text="Password seems to be empty. Exiting now. \nPlease rerun the program and enter a valid password." --ellipsize
                exit 0
            else
            #    touch $HOME/.config/.autoupdate
                echo $MyPass > $HOME/.config/.autoupdate
        zenity --info --timeout=2 --title="Password Remembered" --text="Ok, I got it. I'll remember that password for you." --ellipsize
            fi
    fi
}

dist_cli() {
    usage
    wget -q --method=HEAD https://cdimage.ubuntu.com/ubuntu-mate/releases/$next_LTS/; LTS=$?
        
    if [ "$LTS" == "0" ];then
        zenity --question --title="Bill's Auto-Updater - LTS Check" --text="$next_LTS is available.\n\nWould you like to upgrade now??\n\nIf you're unsure call Bill and set up a time to upgrade your system." --ok-label="Upgrade?" --cancel-label="Not now" --ellipsize; Upgrade=$?
        if [ "$Upgrade" == "0" ];then
            echo "First we need to install all possible updates for the current operating system."
            echo "Installing updates, please wait..."
            
                if [ ! -f "$HOME/.config/.autoupdate" ]; then
                    # Get  or set Password
                    MyPass=$(zenity --password --title="Password Required")
                        if [ -z "$MyPass" ];then
                            zenity --error --title="Password Required" --text="Password seems to be empty. Exiting now. \nPlease rerun the program and enter a valid password." --ellipsize
                            exit 0
                        else
                            touch $HOME/.config/.autoupdate
                            echo $MyPass > $HOME/.config/.autoupdate
                    zenity --info --timeout=2 --title="Password Remembered" --text="Ok, I got it. I'll remember that password for you." --ellipsize
                        fi
                
                else
                    MyPass=$(cat $HOME/.config/.autoupdate)
                fi
            PASSWD="$MyPass"
            echo -e $PASSWD | sudo -S apt-get update
            echo -e $PASSWD | sudo -S apt-get upgrade
            echo -e $MyPass | sudo -S apt-get autoremove
            echo "All updates have been applied"
            echo "Now we'll start the upgrade process"
            zenity --question --title="Upgrading Operating System" --text="Are you sure you want to start the upgrade process?\n\nNOTE: This will take some time and requires the\nsystem to be plugged in." --ellipsize; continue=$?
            if [ "$continue" == "0" ];then
                echo "We'd be doing the upgrade now"
                #echo -e $MyPass | sudo -S do-release-upgrade -d
            else
                exit 1
            fi
        else
            exit 1
        fi
    else
                
        zenity --info --title="Bill's Auto-Updater - LTS Check" --text="\nYour current operating system is version $curr_ver. ($codename)\n\nNo newer LTS version is available at this time.\n\n" --ellipsize
    fi
}

dist() {
#    usage
    wget -q --method=HEAD https://cdimage.ubuntu.com/ubuntu-mate/releases/$next_LTS/; LTS=$?
        
    if [ "$LTS" == "0" ];then
        echo "Ubuntu $next_LTS is ready to download and install. Give it a shot or call Bill."
        newLTS="$next_LTS is available.\n\n"
        timeout="--timeout=30"
    else
        echo "Ubuntu $next_LTS is not ready for release yet. Hang in there, it's coming!"
        newLTS=""
        timeout="--timeout=10"
        #newLTS="You have version $curr_ver.\nNo newer LTS version is available.\n\n"
    fi
}

update() {
    usage
    wget http://$URL/repo/updatectl -O $HOME/updatectl
    PROCEED="$(head -1 $HOME/updatectl)"

    if [ "$PROCEED" -ne "1" ]
    then
        zenity --info --text="Updates have been temporarily disabled by Bill.\nThey will be turned back on when the issues have been fixed.\nThanks for being patient :)" --ellipsize
        exit 0
    fi

    # ~~~~~~~~~~~~~~ ADD ABOVE TO TOP OF FILE (LINES 13 - 22) ~~~~~~~~~~~~~~~~~

    errorfound (){
        if [ "$?" -ne "0" ]
        then
            zenity --info --text="Something went wrong with the automatic update process!\nNeed to check manually or contact Bill.\nQuitting now." --ellipsize
            exit 1
        fi
    }

    if [ ! -f "$HOME/.config/.autoupdate" ]; then
        
        MyPass=$(zenity --password --title="Password Required")
            if [ -z "$MyPass" ];then
                zenity --error --title="Password Required" --text="Password seems to be empty. Exiting now. \nPlease rerun the program and enter a valid password." --ellipsize
                exit 0
            else
                touch $HOME/.config/.autoupdate
                echo $MyPass > $HOME/.config/.autoupdate
        zenity --info --timeout=2 --title="Password Remembered" --text="Ok, I got it. I'll remember that password for you." --ellipsize
            fi
        
    else
        MyPass=$(cat $HOME/.config/.autoupdate)
    fi

    PASSWD="$MyPass"
    MyDate=$(date)

            echo "########################################" 
            echo $MyDate 
            echo "" 
            echo ${txtblu}"Updating package list"${txtrst}
            echo -e $PASSWD | sudo -S apt-get update && sleep 10 
            
            # Check if eggs in installed and remove/install as needed
            apt list --installed | grep -q eggs; Eggs_INST=$?
            if [ "$Eggs_INST" -eq "0" ];then
                apt list --upgradable | grep eggs; EGGS=$?
                if [ "$EGGS" -eq "0" ];then
                    echo "We are going to remove and re-install eggs."
                    echo ""
                    echo -e $PASSWD | sudo -S apt-get remove eggs -y
                    echo ""
                    echo -e $PASSWD | sudo -S apt-get install eggs -y
                    echo ""
                fi
            fi
            errorfound
            echo ${txtblu}"Checking for a newer LTS version of the operating system. Please wait..."${txtrst}
            dist
            echo ${txtblu}"Installing packages. This may take a few minutes, please wait..."${txtrst}
            upGraded=$(echo -e $PASSWD | sudo -S apt-get dist-upgrade --dry-run | grep "not upgraded")
            UPG=$(echo $upGraded | cut -d " " -f 1)
            NEW=$(echo $upGraded | cut -d " " -f 3)
            NOT=$(echo $upGraded | cut -d " " -f 10)
     
            echo -e $PASSWD | sudo -S apt-get dist-upgrade -y
            
            errorfound
            
            echo ${txtblu}"Removing outdated or obsolete packages. This may take a few minutes, please wait..."${txtrst}
            REM=$(echo -e $PASSWD | sudo -S apt-get autoremove --dry-run | grep "to remove" | cut -d " " -f 6)
            echo -e $PASSWD | sudo -S apt-get autoremove -y 
            errorfound
            echo "" 
#            echo "End of update process" 
#            echo "########################################" 
#            echo "" 
            # --timeout=10
            echo "Updates installed successfully"
            echo ""
    zenity --info --text="$UPG packages upgraded.\n$NEW packages newly installed.\n$REM packages removed.\n$NOT packages held back.\n\n$newLTS Updates have been installed successfully!.\nIf there are any problems contact Bill." --ellipsize $timeout
    rm $HOME/updatectl
    
    ### Let's put a check here to see if we want to do the upgrade to new LTS ###
    if [[ -z "$newLTS" ]]; then
        echo "There is no new LTS at this time."
    else
        zenity --question --title="LTS Check" --text="$next_LTS is available.\n\nWould you like to upgrade now?\n\nIf you're unsure call Bill and set up a time to upgrade your system." --ok-label="Upgrade?" --cancel-label="Not now" --ellipsize $timeout; Upgrade=$?
        if [ "$Upgrade" == "0" ];then
            echo "First we need to install all possible updates for the current operating system."
            echo "Installing updates, please wait..."
            
                if [ ! -f "$HOME/.config/.autoupdate" ]; then
                    # Get  or set Password
                    MyPass=$(zenity --password --title="Password Required")
                        if [ -z "$MyPass" ];then
                            zenity --error --title="Password Required" --text="Password seems to be empty. Exiting now. \nPlease rerun the program and enter a valid password." --ellipsize
                            exit 0
                        else
                            touch $HOME/.config/.autoupdate
                            echo $MyPass > $HOME/.config/.autoupdate
                    zenity --info --timeout=2 --title="Password Remembered" --text="Ok, I got it. I'll remember that password for you." --ellipsize
                        fi
                
                else
                    MyPass=$(cat $HOME/.config/.autoupdate)
                fi
            PASSWD="$MyPass"
            #echo -e $PASSWD | sudo -S apt update
            #echo -e $PASSWD | sudo -S apt upgrade
            #echo -e $MyPass | sudo -S apt autoremove
            echo "All updates have been applied"
            echo "Now we'll start the upgrade process"
            zenity --question --title="Upgrading Operating System" --text="Are you sure you want to start the upgrade process?\n\nNOTE: This will take some time and requires the\nsystem to be plugged in." --ellipsize; continue=$?
            if [ "$continue" == "0" ];then
                echo "We'd be doing the upgrade now"
                echo -e $MyPass | sudo -S do-release-upgrade -d
            else
                exit 1
            fi
        else
            exit 1
        fi
    fi
        exit 0
}

pass() {
    MyPass=$(cat $HOME/.config/.autoupdate)
    zenity --info --title="Authorization Credentials" --text="Your password is \"$MyPass\"" --ellipsize
}

curr_version() {
    zenity --info --title="Program Information" --text="\n\nYou have version \"$Version\" of Bill's Auto-Updater.\n\nTo learn more or for other useful scripts visit\n\n<a href=\"https://www.lovinlinux.com/repos.php\">https://www.lovinlinux.com</a>!" --ellipsize
}

next() {

    zenity --info --title="Bill's Auto-Updater - OS Version Information" --text="Your OS is version \"$curr_ver\" ($codename).\n\nThe next available LTS version will be \"$next_LTS\", Codeamed: $next_codename.\n\nHang in there! It's coming in late April of 20$LTSyear" --ellipsize
}

changelog() {
    zenity --info --title="Bill's Auto-Updater ChangeLog" --text="$changes" --ellipsize
}

grab() {
#    curr_ISO=$(curl -L -s https://cdimage.ubuntu.com/ubuntu-mate/releases/ | grep .04 | tail -1 | cut -d "\"" -f 2 | cut -c 1-5)
    
my_choice=$(zenity --list --radiolist --width 350 --height 250 --title="Choose a version" --text="Choose a version to download" --column "" --column "Version" --column "" FALSE "$last_LTS" "Current LTS Release" FALSE "23.04" "Current Stable Release" FALSE "23.10" "Daily Build - For Testing" FALSE "Cancel" "");EC=$?

    if [ "$my_choice" == "23.04" ];then 
        
        curr_isoname=$(curl -L -s https://cdimage.ubuntu.com/ubuntu-mate/releases/$curr_ISO/release/ | grep .iso | grep href= -m 1 | cut -d "\"" -f 2)
        save_to=$(zenity --file-selection --directory --title="Select Save Directory")
        echo "Saving $curr_isoname to $save_to"
        curl -o  "$save_to/$curr_isoname" "https://cdimage.ubuntu.com/ubuntu-mate/releases/$curr_ISO/release/$curr_isoname" && zenity --info --title="File Saved" --text="File successfully saved to\n$save_to/$curr_isoname" --ellipsize; exit 0 || zenity --warning --title="Download failed" --text="Download failed. Please try again" --ellipsize; exit 1
        
    elif [ "$my_choice" == "$last_LTS" ];then 
        
        curr_ltsname=$(curl -L -s https://cdimage.ubuntu.com/ubuntu-mate/releases/$last_LTS/release/ | grep .iso | grep href= -m 1 | cut -d "\"" -f 2)
        save_to=$(zenity --file-selection --directory --title="Select Save Directory")
        echo "Saving $curr_ltsname to $save_to"
        curl -o  "$save_to/$curr_ltsname" "https://cdimage.ubuntu.com/ubuntu-mate/releases/$last_LTS/release/$curr_ltsname" && zenity --info --title="File Saved" --text="File successfully saved to\n$save_to/$curr_ltsname" --ellipsize; exit 0 || zenity --warning --title="Download failed" --text="Download failed. Please try again" --ellipsize; exit 1
    
    elif [ "$my_choice" == "23.10" ];then     
        
        isoname=$(curl -L -s https://cdimage.ubuntu.com/ubuntu-mate/daily-live/current/ | grep .iso | grep href= -m 1 | cut -d "\"" -f 2)
        save_to=$(zenity --file-selection --directory --title="Select Save Directory")
        echo "Saving $isoname to $save_to"
        curl -o "$save_to/$isoname" "https://cdimage.ubuntu.com/ubuntu-mate/daily-live/current/$isoname" && zenity --info --title="File Saved" --text="File successfully saved to\n$save_to/$isoname" --ellipsize; exit 0 || zenity --warning --title="Download failed" --text="Download failed. Please try again" --ellipsize; exit 1
        
    elif [ "$my_choice" == "Cancel" ] || [ "$EC" == "1" ];then

        echo "You've chosen to cancel the download. Nothing has been downloaded."
        zenity --info --title="Bill's Auto-Updater - Download Canceled" --text="The download has been canceled. If you'd like to try again rerun the command \"autoupdate.sh -g\"." 
        exit 0
    fi
}

help() {
    echo ${txtpur}"  #####################################################"
    echo "  ##${txtwht} Bill's Auto-Updater <Author: Bill Swords>       ${txtpur}##"
    echo "  ##${txtwht} https://www.lovinlinux.com                      ${txtpur}##"
#    echo "  ##${txtwht} Version: $Version                                  ${txtpur}##"
    echo "  ##${txtcyn} Usage: autoupdate.sh <option>                   ${txtpur}##"
    echo "  ##${txtrst} Valid options include:                          ${txtpur}##"
    echo "  ##${txtrst} -u|--update  - Update the system                ${txtpur}##"
    echo "  ##${txtrst} -r|--reset   - Reset the password               ${txtpur}##"
    echo "  ##${txtrst} -p|--pass    - View existing password           ${txtpur}##"
    echo "  ##${txtrst} -d|--dist    - Check for newer LTS OS version   ${txtpur}##"
    echo "  ##${txtrst} -n|--next    - See next LTS OS version number   ${txtpur}##"
    echo "  ##${txtrst} -c|--changes - View the changelog               ${txtpur}##"
    echo "  ##${txtrst} -v|--version - View version info                ${txtpur}##"
    echo "  ##${txtrst} -g|--grab    - Grab latest available ISO        ${txtpur}##"
    echo "  ##${txtrst} -h|--help    - View this help information       ${txtpur}##"
    echo "  #####################################################"${txtrst}
}

case $1 in
    -r|--reset)
        reset_pass
        ;;
    -u|--update)
        update
        ;;
    -p|--pass)
        pass
        ;;
    -d|--dist)
        dist_cli
        ;;
    -n|--next)
        next
        ;;
    -g|--grab)
        grab
        ;;
    -h|--help)
        help
        ;;
    -v|--version)
        curr_version
        ;;    
    -c|changes)
        changelog
        ;;
    *)
        update
        ;;
esac
